@@ -2,6 +2,9 @@
     begin                : Mon Feb  4 23:40:41 EST 2002
     copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
+
+    copyright            : (C) 2008 by Michael Pyne
+    email                : michael.pyne@kdemail.net
 ***************************************************************************/
 
 /***************************************************************************
@@ -34,6 +37,7 @@
 #include <ktoolbarpopupaction.h>
 #include <kdeversion.h>
 
+#include <QCoreApplication>
 #include <QKeyEvent>
 #include <QDir>
 #include <QTimer>
@@ -295,7 +299,8 @@ void JuK::setupSystemTray()
         m_toggleDockOnCloseAction->setEnabled(true);
         m_togglePopupsAction->setEnabled(true);
 
-        connect(m_systemTray, SIGNAL(quitSelected()), this, SLOT(slotAboutToQuit()));
+        connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), SLOT(slotAboutToQuit()));
+        connect(m_systemTray, SIGNAL(quitSelected()), this, SLOT(slotQuit()));
     }
     else {
         m_systemTray = 0;
@@ -442,21 +447,13 @@ void JuK::saveConfig()
 
 bool JuK::queryExit()
 {
+    // This may actually be sent more than once in practice, but we're not
+    // supposed to do end of execution destruction yet anyways, use
+    // slotAboutToQuit for that.
+
     m_startDocked = !isVisible();
-
-    kDebug(65432) ;
-
-    hide();
-
-    action("stop")->trigger();
-    delete m_systemTray;
-    m_systemTray = 0;
-
-    CoverManager::shutdown();
-    Cache::instance()->save();
     saveConfig();
 
-    delete m_splitter;
     return true;
 }
 
@@ -492,6 +489,17 @@ void JuK::slotShowHide()
 void JuK::slotAboutToQuit()
 {
     m_shuttingDown = true;
+
+    action("stop")->trigger();
+    delete m_systemTray;
+    m_systemTray = 0;
+
+    CoverManager::shutdown();
+    Cache::instance()->save();
+    saveConfig();
+
+    delete m_splitter;
+    m_splitter = 0;
 }
 
 void JuK::slotQuit()
