@@ -734,7 +734,17 @@ void Playlist::synchronizePlayingItems(const PlaylistList &sources, bool setMast
 
 void Playlist::copy()
 {
-    QApplication::clipboard()->setMimeData(drag(0)->mimeData(), QClipboard::Clipboard);
+    PlaylistItemList items = selectedItems();
+    KUrl::List urls;
+
+    foreach(PlaylistItem *item, items) {
+        urls << KUrl::fromPath(item->file().absFilePath());
+    }
+
+    QMimeData *mimeData = new QMimeData;
+    urls.populateMimeData(mimeData);
+
+    QApplication::clipboard()->setMimeData(mimeData, QClipboard::Clipboard);
 }
 
 void Playlist::paste()
@@ -830,27 +840,22 @@ void Playlist::slotAddCover(bool retrieveLocal)
     if(items.isEmpty())
         return;
 
-    QPixmap newCover;
-
-    if(retrieveLocal) {
-        KUrl file = KFileDialog::getImageOpenUrl(
-            KUrl( "kfiledialog://homedir" ), this, i18n("Select Cover Image File"));
-        newCover = QPixmap(file.directory() + '/' + file.fileName());
-    }
-    else {
+    if(!retrieveLocal) {
         m_fetcher->setFile((*items.begin())->file());
         m_fetcher->chooseCover();
         return;
     }
 
-    if(newCover.isNull())
-        return;
+    KUrl file = KFileDialog::getImageOpenUrl(
+        KUrl( "kfiledialog://homedir" ), this, i18n("Select Cover Image File"));
 
     QString artist = items.front()->file().tag()->artist();
     QString album = items.front()->file().tag()->album();
 
-    coverKey newId = CoverManager::addCover(newCover, artist, album);
-    refreshAlbums(items, newId);
+    coverKey newId = CoverManager::addCover(file, artist, album);
+
+    if(newId != CoverManager::NoMatch)
+        refreshAlbums(items, newId);
 }
 
 // Called when image fetcher has added a new cover.
@@ -999,24 +1004,20 @@ void Playlist::removeFromDisk(const PlaylistItemList &items)
     }
 }
 
-QDrag *Playlist::drag(QWidget *parent)
+Q3DragObject *Playlist::dragObject(QWidget *parent)
 {
     PlaylistItemList items = selectedItems();
     KUrl::List urls;
-    for(PlaylistItemList::Iterator it = items.begin(); it != items.end(); ++it) {
-        KUrl url;
-        url.setPath((*it)->file().absFilePath());
-        urls.append(url);
+
+    foreach(PlaylistItem *item, items) {
+        urls << KUrl::fromPath(item->file().absFilePath());
     }
 
-    QDrag *drag = new QDrag( parent );
-    QMimeData *md = new QMimeData;
-    drag->setMimeData( md );
-    urls.populateMimeData( md );
+    K3URLDrag *urlDrag = new K3URLDrag(urls, parent);
 
-    drag->setPixmap(BarIcon("sound"));
+    urlDrag->setPixmap(BarIcon("sound"));
 
-    return drag;
+    return urlDrag;
 }
 
 void Playlist::contentsDragEnterEvent(QDragEnterEvent *e)
@@ -1049,16 +1050,6 @@ bool Playlist::acceptDrag(QDropEvent *e) const
     return CoverDrag::canDecode(e) || K3URLDrag::canDecode(e);
 }
 
-bool Playlist::canDecode(QMimeSource *s)
-{
-    KUrl::List urls;
-
-    if(CoverDrag::canDecode(s))
-        return true;
-
-    return K3URLDrag::decode(s, urls) && !urls.isEmpty();
-}
-
 void Playlist::decode(const QMimeData *s, PlaylistItem *item)
 {
     KUrl::List urls;
