@@ -49,6 +49,8 @@
 #include "ksmallslider.h"
 #include "verticaltext.h"
 
+static const int MIN_SLIDER_SIZE = 150;
+
 /**
  * MixDeviceWidget that represents a single mix device, inlcuding PopUp, muteLED, ...
  *
@@ -169,9 +171,13 @@ void MDWSlider::setPlaybackExtent(int extent) {
         if ( _orientation == Qt::Vertical )
              if ( playbackExtentHint() < extent )
                m_playbackSpacer->setFixedHeight(extent-playbackExtentHint());
+             else
+               m_playbackSpacer->setFixedHeight(0);
         else
             if ( playbackExtentHint() < extent )
                m_playbackSpacer->setFixedWidth(extent-playbackExtentHint());
+             else
+               m_playbackSpacer->setFixedWidth(0);
     }
 }
 
@@ -198,11 +204,15 @@ void MDWSlider::setCaptureExtent(int extent) {
         m_defaultLabelSpacer->setFixedHeight(extent);
         if ( captureExtentHint() < extent )
             m_captureSpacer->setFixedHeight(extent-captureExtentHint());
+        else
+            m_captureSpacer->setFixedHeight(0);
     }
     else {
         m_defaultLabelSpacer->setFixedWidth(extent);
         if ( captureExtentHint() < extent )
             m_captureSpacer->setFixedWidth(extent-captureExtentHint());
+        else
+            m_captureSpacer->setFixedWidth(0);
     }
 }
 
@@ -459,6 +469,13 @@ void MDWSlider::addSliders( QBoxLayout *volLayout, char type, bool addLabel)
             sliderBig->setMaximum(maxvol);
             sliderBig->setPageStep(maxvol/10);
             sliderBig->setValue(maxvol - vol.getVolume( chid ));
+
+            if ( _orientation == Qt::Vertical ) {
+                sliderBig->setMinimumHeight( MIN_SLIDER_SIZE );
+            }
+            else {
+                sliderBig->setMinimumWidth( MIN_SLIDER_SIZE );
+            }
         } // not small
 
         slider->installEventFilter( this );
